@@ -197,8 +197,6 @@ yelp_docbook_document_new (YelpUri *uri)
 
     priv->uri = g_object_ref (uri);
 
-    yelp_document_set_page_id (YELP_DOCUMENT (docbook), "//about", "//about");
-
     priv->sections =
         GTK_TREE_MODEL (gtk_tree_store_new (2, G_TYPE_STRING, G_TYPE_STRING));
 
@@ -356,17 +354,13 @@ docbook_process (YelpDocbookDocument *docbook)
         id = xmlGetNsProp (priv->xmlcur, XML_XML_NAMESPACE, BAD_CAST "id");
 
     if (id) {
-        priv->root_id = g_strdup (id);
+        priv->root_id = g_strdup ((const gchar *) id);
         yelp_document_set_page_id (document, NULL, (gchar *) id);
         yelp_document_set_page_id (document, "//index", (gchar *) id);
-        yelp_document_set_prev_id (document, (gchar *) id, "//about");
-        yelp_document_set_next_id (document, "//about", (gchar *) id);
     }
     else {
         priv->root_id = g_strdup ("//index");
         yelp_document_set_page_id (document, NULL, "//index");
-        yelp_document_set_prev_id (document, "//index", "//about");
-        yelp_document_set_next_id (document, "//about", "//index");
         /* add the id attribute to the root element with value "index"
          * so when we try to load the document later, it doesn't fail */
         if (priv->xmlcur->ns)
@@ -377,7 +371,6 @@ docbook_process (YelpDocbookDocument *docbook)
             xmlNewProp (priv->xmlcur, BAD_CAST "id", BAD_CAST "//index");
     }
     yelp_document_set_root_id (document, priv->root_id, priv->root_id);
-    yelp_document_set_root_id (document, "//about", priv->root_id);
     g_mutex_unlock (priv->mutex);
 
     g_mutex_lock (priv->mutex);
@@ -395,6 +388,8 @@ docbook_process (YelpDocbookDocument *docbook)
         goto done;
     }
 
+    priv->state = DOCBOOK_STATE_PARSED;
+
     priv->transform = yelp_transform_new (STYLESHEET);
     priv->chunk_ready =
         g_signal_connect (priv->transform, "chunk-ready",
