@@ -32,6 +32,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
+#include "yelp-bookmarks.h"
 #include "yelp-settings.h"
 #include "yelp-view.h"
 
@@ -43,7 +44,6 @@
 static gboolean editor_mode = FALSE;
 
 enum {
-    BOOKMARKS_CHANGED,
     READ_LATER_CHANGED,
     LAST_SIGNAL
 };
@@ -73,10 +73,11 @@ static const gchar introspection_xml[] =
     "</node>";
 GDBusNodeInfo *introspection_data;
 
-static void          yelp_application_init             (YelpApplication       *app);
-static void          yelp_application_class_init       (YelpApplicationClass  *klass);
-static void          yelp_application_dispose          (GObject               *object);
-static void          yelp_application_finalize         (GObject               *object);
+static void          yelp_application_init             (YelpApplication        *app);
+static void          yelp_application_class_init       (YelpApplicationClass   *klass);
+static void          yelp_application_iface_init       (YelpBookmarksInterface *iface);
+static void          yelp_application_dispose          (GObject                *object);
+static void          yelp_application_finalize         (GObject                *object);
 
 static void          yelp_application_method           (GDBusConnection       *connection,
                                                         const gchar           *sender,
@@ -100,10 +101,18 @@ static void          application_adjust_font           (GtkAction             *a
                                                         YelpApplication       *app);
 static void          application_set_font_sensitivity  (YelpApplication       *app);
 
+static void          bookmarks_changed                 (GSettings             *settings,
+                                                        const gchar           *key,
+                                                        YelpApplication       *app);
+static void          readlater_changed                 (GSettings             *settings,
+                                                        const gchar           *key,
+                                                        YelpApplication       *app);
 static gboolean      window_resized                    (YelpWindow            *window,
                                                         YelpApplication       *app);
 
-G_DEFINE_TYPE (YelpApplication, yelp_application, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_CODE (YelpApplication, yelp_application, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (YELP_TYPE_BOOKMARKS,
+                                                yelp_application_iface_init))
 #define GET_PRIV(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), YELP_TYPE_APPLICATION, YelpApplicationPrivate))
 
 GDBusInterfaceVTable yelp_dbus_vtable = {
@@ -133,7 +142,7 @@ static const GtkActionEntry action_entries[] = {
     { "SmallerText", GTK_STOCK_ZOOM_OUT,
       N_("_Smaller Text"),
       "<Control>minus",
-      N_("Descrease the size of the text"),
+      N_("Decrease the size of the text"),
       G_CALLBACK (application_adjust_font) }
 };
 
@@ -154,14 +163,6 @@ yelp_application_class_init (YelpApplicationClass *klass)
     object_class->dispose = yelp_application_dispose;
     object_class->finalize = yelp_application_finalize;
 
-    signals[BOOKMARKS_CHANGED] =
-        g_signal_new ("bookmarks-changed",
-                      G_TYPE_FROM_CLASS (klass),
-                      G_SIGNAL_RUN_LAST,
-                      0, NULL, NULL,
-                      g_cclosure_marshal_VOID__STRING,
-                      G_TYPE_NONE, 1, G_TYPE_STRING);
-
     signals[READ_LATER_CHANGED] =
         g_signal_new ("read-later-changed",
                       G_TYPE_FROM_CLASS (klass),
@@ -173,6 +174,14 @@ yelp_application_class_init (YelpApplicationClass *klass)
     g_type_class_add_private (klass, sizeof (YelpApplicationPrivate));
 }
 
+static void
+yelp_application_iface_init (YelpBookmarksInterface *iface)
+{
+    iface->add_bookmark = yelp_application_add_bookmark;
+    iface->remove_bookmark = yelp_application_remove_bookmark;
+    iface->is_bookmarked = yelp_application_is_bookmarked;
+}
+
 static void
 yelp_application_dispose (GObject *object)
 {
@@ -564,8 +573,7 @@ application_get_doc_settings (YelpApplication *app, const gchar *doc_uri)
     YelpApplicationPrivate *priv = GET_PRIV (app);
     GSettings *settings = g_hash_table_lookup (priv->docsettings, doc_uri);
     if (settings == NULL) {
-        gchar *tmp;
-        gchar *settings_path;
+        gchar *tmp, *key, *settings_path;
         tmp = g_uri_escape_string (doc_uri, "", FALSE);
         settings_path = g_strconcat ("/apps/yelp/documents/", tmp, "/", NULL);
         g_free (tmp);
@@ -576,7 +584,13 @@ application_get_doc_settings (YelpApplication *app, const gchar *doc_uri)
         else
             settings = g_settings_new_with_path ("org.gnome.yelp.document",
                                                  settings_path);
-        g_hash_table_insert (priv->docsettings, g_strdup (doc_uri), settings);
+        key = g_strdup (doc_uri);
+        g_hash_table_insert (priv->docsettings, key, settings);
+        g_object_set_data ((GObject *) settings, "doc_uri", key);
+        g_signal_connect (settings, "changed::bookmarks",
+                          G_CALLBACK (bookmarks_changed), app);
+        g_signal_connect (settings, "changed::readlater",
+                          G_CALLBACK (readlater_changed), app);
         g_free (settings_path);
     }
     return settings;
@@ -615,13 +629,14 @@ application_maybe_quit (YelpApplication *app)
 /******************************************************************************/
 
 void
-yelp_application_add_bookmark (YelpApplication   *app,
+yelp_application_add_bookmark (YelpBookmarks     *bookmarks,
                                const gchar       *doc_uri,
                                const gchar       *page_id,
                                const gchar       *icon,
                                const gchar       *title)
 {
     GSettings *settings;
+    YelpApplication *app = YELP_APPLICATION (bookmarks);
 
     settings = application_get_doc_settings (app, doc_uri);
 
@@ -647,17 +662,20 @@ yelp_application_add_bookmark (YelpApplication   *app,
             g_variant_builder_add (&builder, "(sss)", page_id, icon, title);
             value = g_variant_builder_end (&builder);
             g_settings_set_value (settings, "bookmarks", value);
-            g_signal_emit (app, signals[BOOKMARKS_CHANGED], 0, doc_uri);
         }
     }
 }
 
 void
-yelp_application_remove_bookmark (YelpApplication   *app,
+yelp_application_remove_bookmark (YelpBookmarks     *bookmarks,
                                   const gchar       *doc_uri,
                                   const gchar       *page_id)
 {
     GSettings *settings;
+    YelpApplication *app = YELP_APPLICATION (bookmarks);
+
+    g_return_if_fail (page_id);
+    g_return_if_fail (doc_uri);
 
     settings = application_get_doc_settings (app, doc_uri);
 
@@ -674,10 +692,42 @@ yelp_application_remove_bookmark (YelpApplication   *app,
         g_variant_iter_free (iter);
 
         g_settings_set_value (settings, "bookmarks", g_variant_builder_end (&builder));
-        g_signal_emit (app, signals[BOOKMARKS_CHANGED], 0, doc_uri);
     }
 }
 
+gboolean
+yelp_application_is_bookmarked (YelpBookmarks     *bookmarks,
+                                const gchar       *doc_uri,
+                                const gchar       *page_id)
+{
+    GVariant *stored_bookmarks;
+    GVariantIter *iter;
+    gboolean ret = FALSE;
+    gchar *this_id = NULL;
+    GSettings *settings;
+    YelpApplication *app = YELP_APPLICATION (bookmarks);
+
+    g_return_val_if_fail (page_id, FALSE);
+    g_return_val_if_fail (doc_uri, FALSE);
+
+    settings = application_get_doc_settings (app, doc_uri);
+    if (settings == NULL)
+        return FALSE;
+
+    stored_bookmarks = g_settings_get_value (settings, "bookmarks");
+    g_settings_get (settings, "bookmarks", "a(sss)", &iter);
+    while (g_variant_iter_loop (iter, "(&s&s&s)", &this_id, NULL, NULL)) {
+        if (g_str_equal (page_id, this_id)) {
+            ret = TRUE;
+            break;
+        }
+    }
+
+    g_variant_iter_free (iter);
+    g_variant_unref (stored_bookmarks);
+    return ret;
+}
+
 void
 yelp_application_update_bookmarks (YelpApplication   *app,
                                    const gchar       *doc_uri,
@@ -687,6 +737,9 @@ yelp_application_update_bookmarks (YelpApplication   *app,
 {
     GSettings *settings;
 
+    g_return_if_fail (page_id);
+    g_return_if_fail (doc_uri);
+
     settings = application_get_doc_settings (app, doc_uri);
 
     if (settings) {
@@ -717,7 +770,6 @@ yelp_application_update_bookmarks (YelpApplication   *app,
             GVariant *value;
             value = g_variant_builder_end (&builder);
             g_settings_set_value (settings, "bookmarks", value);
-            g_signal_emit (app, signals[BOOKMARKS_CHANGED], 0, doc_uri);
         }
     }
 }
@@ -731,33 +783,6 @@ yelp_application_get_bookmarks (YelpApplication *app,
     return g_settings_get_value (settings, "bookmarks");
 }
 
-static void
-packages_installed (GDBusConnection *connection,
-                    GAsyncResult *res,
-                    YelpApplication *app)
-{
-    GError *error = NULL;
-    g_dbus_connection_call_finish (connection, res, &error);
-    if (error) {
-        const gchar *err = NULL;
-        if (error->domain == G_DBUS_ERROR) {
-            if (error->code == G_DBUS_ERROR_SERVICE_UNKNOWN)
-                err = _("You do not have PackageKit installed.  Package installation links require PackageKit.");
-            else
-                err = error->message;
-        }
-        if (err != NULL) {
-            GtkWidget *dialog = gtk_message_dialog_new (NULL, 0,
-                                                        GTK_MESSAGE_ERROR,
-                                                        GTK_BUTTONS_CLOSE,
-                                                        "%s", err);
-            gtk_dialog_run ((GtkDialog *) dialog);
-            gtk_widget_destroy (dialog);
-        }
-        g_error_free (error);
-    }
-}
-
 void
 yelp_application_add_read_later (YelpApplication   *app,
                                  const gchar       *doc_uri,
@@ -790,7 +815,6 @@ yelp_application_add_read_later (YelpApplication   *app,
             g_variant_builder_add (&builder, "(ss)", full_uri, title);
             value = g_variant_builder_end (&builder);
             g_settings_set_value (settings, "readlater", value);
-            g_signal_emit (app, signals[READ_LATER_CHANGED], 0, doc_uri);
         }
     }
 }
@@ -817,7 +841,6 @@ yelp_application_remove_read_later (YelpApplication *app,
         g_variant_iter_free (iter);
 
         g_settings_set_value (settings, "readlater", g_variant_builder_end (&builder));
-        g_signal_emit (app, signals[READ_LATER_CHANGED], 0, doc_uri);
     }
 }
 
@@ -830,28 +853,24 @@ yelp_application_get_read_later (YelpApplication *app,
     return g_settings_get_value (settings, "readlater");
 }
 
-void
-yelp_application_install_package (YelpApplication  *app,
-                                  const gchar      *pkg,
-                                  const gchar      *alt)
+static void
+bookmarks_changed (GSettings       *settings,
+                   const gchar     *key,
+                   YelpApplication *app)
 {
-    GVariantBuilder *strv;
-    YelpApplicationPrivate *priv = GET_PRIV (app);
-    guint32 xid = 0;
-    strv = g_variant_builder_new (G_VARIANT_TYPE ("as"));
-    g_variant_builder_add (strv, "s", pkg);
-    g_dbus_connection_call (priv->connection,
-                            "org.freedesktop.PackageKit",
-                            "/org/freedesktop/PackageKit",
-                            "org.freedesktop.PackageKit.Modify",
-                            "InstallPackageNames",
-                            g_variant_new ("(uass)", xid, strv, ""),
-                            NULL,
-                            G_DBUS_CALL_FLAGS_NONE,
-                            -1, NULL,
-                            (GAsyncReadyCallback) packages_installed,
-                            app);
-    g_variant_builder_unref (strv);
+    const gchar *doc_uri = g_object_get_data ((GObject *) settings, "doc_uri");
+    if (doc_uri)
+        g_signal_emit_by_name (app, "bookmarks-changed", doc_uri);
+}
+
+static void
+readlater_changed (GSettings       *settings,
+                   const gchar     *key,
+                   YelpApplication *app)
+{
+    const gchar *doc_uri = g_object_get_data ((GObject *) settings, "doc_uri");
+    if (doc_uri)
+        g_signal_emit (app, signals[READ_LATER_CHANGED], 0, doc_uri);
 }
 
 static gboolean
